function Test-Command 
{
    Param(
        [string]$command
    )

    if (Get-Command $command -ErrorAction Ignore) 
    {
        return $true
    }
    else 
    {
        return $false
    }
}

function global:Get-XdeDeviceList 
{
    Param(
        [string]$Name = ""
    )

    $devicelist = @{}

    if ("$(xdeconfig list)" -eq "") 
    {
        return $devicelist
    }

    $listoutput = xdeconfig list $Name -c
    if ($listoutput[0].GetType() -eq [System.Char])
    {
        return $devicelist
    }

    $schema = $listoutput[0].Split(",")

    foreach ($rowindex in 1..($listoutput.Count - 1)) 
    {
        $rowsplit = $listoutput[$rowindex].Split(",")
        if ($rowsplit.Count -ne $schema.Count) 
        {
            Write-Error "'xdeconfig list -c' first row count != count for row $rowindex"
        }

        $device = @{ }
        foreach ($i in 0..($rowsplit.Count - 1)) 
        {
            $value = $rowsplit[$i]
            if ($value -eq "True")
            {
                $value = $true
            }
            elseif ($value -eq "False")
            {
                $value = $false
            }

            $device[$schema[$i]] = $value

            if ($schema[$i] -eq "VHDSource") {
                $split = $device.VHDSource.Split("\")
                $device.branch = $split[-9]
                $device.build = $split[-8]
                $device.imagetype = $split[-3]
            }
        }

        $devicelist[$device.name] = $device
    }

    return $devicelist
}
Export-ModuleMember Get-XdeDeviceList

function global:Get-XdeDevice 
{
    Param(
        [Parameter(Mandatory=$true)]
        [string]$Name
    )

    $devicelist = Get-XdeDeviceList -Name $Name
    if ($devicelist.count -eq 1)
    {
        return $devicelist[$devicelist.Keys[0]];
    }

    throw [System.Management.Automation.ItemNotFoundException] "Could not find XDE device: $Name"
}
Export-ModuleMember Get-XdeDevice

function global:Open-XdeDevice 
{
    Param(
        [Parameter(Mandatory=$true)]
        [string]$Name
    )

    $device = Get-XdeDevice $Name
    if ($device.IsRunning -eq $false)
    {
        throw [System.InvalidOperationException] "The $Name device isn't running."
    }

    if ($device.IPAddress -eq "")
    {
        throw [System.InvalidOperationException] "The $Name device doesn't yet have an IP address."
    }

    if (!(Test-Command "open-device")) 
    {
        Write-Error "The tshell commands weren't found. Make sure you've installed tshell and that you're running the x86 version PowerShell."
        return
    }

    open-device $device.IPAddress
}
Export-ModuleMember Open-XdeDevice

#
# Start-Xde (xdeconfig launch)
#
function global:Start-Xde 
{
    Param(
        [Parameter(Mandatory=$true)]
        [string]$Name
    )

    xdeconfig launch $Name
}
Export-ModuleMember Start-Xde

#
# Start-XdeDebug (xdeconfig kdebug)
#
function global:Start-XdeDebug 
{
    Param(
        [Parameter(Mandatory=$true)]
        [string]$Name
    )

    xdeconfig kdebug -d $Name
}
Export-ModuleMember Start-XdeDebug

#
# Reset-XdeDevice (xdeconfig reset/ redownload)
#
function global:Reset-XdeDevice 
{
    Param(
        [Parameter(Mandatory=$true)]
        [string]$Name
    )

    $DeviceObj = Get-XdeDevice $Name

    if ($DeviceObj.UseDiffDisk) 
    {
        xdeconfig reset -d $Name
    }
    else 
    {
        xdeconfig redownload -d $Name
    }
}
Export-ModuleMember Reset-XdeDevice

#
# Delete-XdeDevice (xdeconfig delete)
#
function global:Delete-XdeDevice 
{
    Param(
        [Parameter(Mandatory=$true)]
        [string]$Name
    )

    xdeconfig delete -d $Name
}
Export-ModuleMember Delete-XdeDevice

#
# Delete-AllXdeDevices (xdepurge)
# Deletes ALL existing devices (xdeconfig delete)
#
function global:Delete-AllXdeDevices 
{

    $list = Get-XdeDeviceList
    foreach ($device in $list.Keys) 
    { 
        xdeconfig delete -d $device 
    }
}
Export-ModuleMember Delete-AllXdeDevices
